@@ -4,7 +4,6 @@ import (
 	"context"
 	"fmt"
 	"reflect"
-	"strconv"
 	"strings"
 	"time"
 
@@ -19,6 +18,7 @@ import (
 	utilerrors "k8s.io/apimachinery/pkg/util/errors"
 	"k8s.io/client-go/dynamic"
 	"k8s.io/client-go/kubernetes"
+	"k8s.io/client-go/util/retry"
 	"k8s.io/klog/v2"
 
 	workv1client "github.com/open-cluster-management/api/client/work/clientset/versioned/typed/work/v1"
@@ -145,7 +145,16 @@ func (m *ManifestWorkController) sync(ctx context.Context, controllerContext fac
 
 	errs := []error{}
 	// Apply resources on spoke cluster.
-	resourceResults := m.applyManifest(manifestWork.Spec.Workload.Manifests, controllerContext.Recorder(), *owner)
+	resourceResults := make([]resourceapply.ApplyResult, len(manifestWork.Spec.Workload.Manifests))
+	retry.RetryOnConflict(retry.DefaultBackoff, func() error {
+		resourceResults = m.applyManifests(manifestWork.Spec.Workload.Manifests, controllerContext.Recorder(), *owner, resourceResults)
+		for _, result := range resourceResults {
+			if errors.IsConflict(result.Error) {
+				return result.Error
+			}
+		}
+		return nil
+	})
 	newManifestConditions := []workapiv1.ManifestCondition{}
 	for index, result := range resourceResults {
 		if result.Error != nil {
@@ -181,36 +190,47 @@ func (m *ManifestWorkController) sync(ctx context.Context, controllerContext fac
 	return err
 }
 
-func (m *ManifestWorkController) applyManifest(manifests []workapiv1.Manifest, recorder events.Recorder, owner metav1.OwnerReference) []resourceapply.ApplyResult {
+func (m *ManifestWorkController) applyManifests(
+	manifests []workapiv1.Manifest, recorder events.Recorder, owner metav1.OwnerReference, existingResults []resourceapply.ApplyResult) []resourceapply.ApplyResult {
+	for index, manifest := range manifests {
+		switch {
+		case existingResults[index].Result == nil:
+			// Apply if there is not result.
+			existingResults[index] = m.applyOneManifest(manifest, recorder, owner)
+		case errors.IsConflict(existingResults[index].Error):
+			// Apply if there is a resource confilct error.
+			existingResults[index] = m.applyOneManifest(manifest, recorder, owner)
+		}
+	}
+
+	return existingResults
+}
+
+func (m *ManifestWorkController) applyOneManifest(
+	manifest workapiv1.Manifest, recorder events.Recorder, owner metav1.OwnerReference) resourceapply.ApplyResult {
 	clientHolder := resourceapply.NewClientHolder().
 		WithAPIExtensionsClient(m.spokeAPIExtensionClient).
-		WithKubernetes(m.spokeKubeclient)
+		WithKubernetes(m.spokeKubeclient).
+		WithDynamicClient(m.spokeDynamicClient)
 
-	// Using index as the file name and apply manifests
-	indexstrings := []string{}
-	for i := 0; i < len(manifests); i++ {
-		indexstrings = append(indexstrings, strconv.Itoa(i))
-	}
 	results := resourceapply.ApplyDirectly(clientHolder, recorder, func(name string) ([]byte, error) {
-		index, _ := strconv.ParseInt(name, 10, 32)
 		unstructuredObj := &unstructured.Unstructured{}
-		err := unstructuredObj.UnmarshalJSON(manifests[index].Raw)
+		err := unstructuredObj.UnmarshalJSON(manifest.Raw)
 		if err != nil {
 			return nil, err
 		}
 		unstructuredObj.SetOwnerReferences([]metav1.OwnerReference{owner})
 		return unstructuredObj.MarshalJSON()
-	}, indexstrings...)
+	}, "manifest")
 
 	// Try apply with dynamic client if the manifest cannot be decoded by scheme or typed client is not found
 	// TODO we should check the certain error.
-	for index, result := range results {
-		// Use dynamic client when scheme cannot decode manifest or typed client cannot handle the object
-		if isDecodeError(result.Error) || isUnhandledError(result.Error) {
-			results[index].Result, results[index].Changed, results[index].Error = m.applyUnstructrued(manifests[index].Raw, owner, recorder)
-		}
+	// Use dynamic client when scheme cannot decode manifest or typed client cannot handle the object
+	result := results[0]
+	if isDecodeError(result.Error) || isUnhandledError(result.Error) || isUnsupportedError(result.Error) {
+		result.Result, result.Changed, result.Error = m.applyUnstructrued(manifest.Raw, owner, recorder)
 	}
-	return results
+	return result
 }
 
 func (m *ManifestWorkController) decodeUnstructured(data []byte) (schema.GroupVersionResource, *unstructured.Unstructured, error) {
@@ -295,7 +315,7 @@ func (m *ManifestWorkController) generateUpdateStatusFunc(newManifestConditions
 	}
 }
 
-// isDecodeError is to check if the error returned from resourceapply is due to that the object cannnot
+// isDecodeError is to check if the error returned from resourceapply is due to that the object cannot
 // be decoded or no typed client can handle the object.
 func isDecodeError(err error) bool {
 	return err != nil && strings.HasPrefix(err.Error(), "cannot decode")
@@ -307,13 +327,19 @@ func isUnhandledError(err error) bool {
 	return err != nil && strings.HasPrefix(err.Error(), "unhandled type")
 }
 
+// isUnsupportedError is to check if the error returned from resourceapply is due to
+// the PR https://github.com/openshift/library-go/pull/1042
+func isUnsupportedError(err error) bool {
+	return err != nil && strings.HasPrefix(err.Error(), "unsupported object type")
+}
+
 // isSameUnstructured compares the two unstructured object.
-// The comparison ignores the metadata and status field, and check if the two objects are sementically equal.
+// The comparison ignores the metadata and status field, and check if the two objects are semantically equal.
 func isSameUnstructured(obj1, obj2 *unstructured.Unstructured) bool {
 	obj1Copy := obj1.DeepCopy()
 	obj2Copy := obj2.DeepCopy()
 
-	// Comapre gvk, name, namespace at first
+	// Compare gvk, name, namespace at first
 	if obj1Copy.GroupVersionKind() != obj2Copy.GroupVersionKind() {
 		return false
 	}
@@ -332,7 +358,7 @@ func isSameUnstructured(obj1, obj2 *unstructured.Unstructured) bool {
 		return false
 	}
 
-	// Compare sementically after removing metadata and status field
+	// Compare semantically after removing metadata and status field
 	delete(obj1Copy.Object, "metadata")
 	delete(obj2Copy.Object, "metadata")
 	delete(obj1Copy.Object, "status")
